/* Copyright 2019 Rozakin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include QMK_KEYBOARD_H

// Each layer gets a name for readability, which is then used in the keymap matrix below.
// The underscores don't mean anything - you can have a layer called STUFF or any other name.
// Layer names don't all need to be of the same length, obviously, and you can also skip them
// entirely and just use numbers.
#define _BL 0
#define _FL1 1
#define _FL2 2
#define _FL3 3

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
    /* _BL: Base Layer(Default) - For ISO enter use ANSI enter
    * ,------------------------------------------------------------.  ,--------------.
    * |Esc  |f1| f2| f3| f4|  | f5| f6| f7| f8|   | f9|f10|f11| f12|  |Prnt|ScLk|Paus|
    * |------------------------------------------------------------|  |--------------|
    * | ~ |  1|  2|  3|  4|  5|  6|  7|  8|  9|  0|  -|  =|  \|BSpc|  | Ins|Home|PgUp|
    * |------------------------------------------------------------|  |--------------|
    * |Tab  |  Q|  W|  E|  R|  T|  Y|  U|  I|  O|  P|  [|  ]|   \  |  | Del| End|PgDn|
    * |------------------------------------------------------------|  `--------------'
    * |CAPS   |  A|  S|  D|  F|  G|  H|  J|  K|  L|  ;|  '| #|  Ent|
    * |------------------------------------------------------------|       ,----.
    * |Shift| \|  Z|  X|  C|  V|  B|  N|  M|  ,|  .|  /|Shift|Mo(1)|       | Up |
    * |------------------------------------------------------------|  ,--------------.
    * |Ctrl|Win |Alt |        Space         |Alt |Win |Mo(1) |Ctrl |  |Left| Dn |Rght|
    * `------------------------------------------------------------'  `--------------'
    */
    [_BL] = LAYOUT(
        KC_ESC,           KC_F1,   KC_F2,  KC_F3,   KC_F4, KC_F5,   KC_F6,   KC_F7,   KC_F8,            KC_F9,   KC_F10,   KC_F11,  KC_F12,    KC_PSCR,  KC_SLCK,  KC_PAUS,
        KC_GRV,  KC_1,    KC_2,    KC_3,   KC_4,    KC_5,  KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS, KC_EQL,   KC_BSLS, KC_BSPC,   KC_INS,   KC_HOME,  KC_PGUP,
        KC_TAB,  KC_Q,    KC_W,    KC_E,   KC_R,    KC_T,  KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_LBRC, KC_RBRC,           KC_BSLS,   KC_DEL,   KC_END,   KC_PGDN,
        KC_CAPS, KC_A,    KC_S,    KC_D,   KC_F,    KC_G,  KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT, KC_NUHS,           KC_ENT,
        KC_LSFT, KC_NUBS, KC_Z,    KC_X,   KC_C,    KC_V,  KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, KC_RSFT,           MO(_FL1),            KC_UP,
        KC_LCTL, KC_LGUI, KC_LALT,                         KC_SPC,                             KC_RALT, KC_RGUI, MO(_FL1),          KC_RCTRL,   KC_LEFT,  KC_DOWN,  KC_RGHT),
        /* _FL1: Function Layer 1 - For ISO enter use ANSI enter
        * ,------------------------------------------------------------.  ,--------------.
        * |     |  |   |   |   |  |   |   |   |   |   |   |   |   |    |  |    |    |    |
        * |------------------------------------------------------------|  |--------------|
        * |   |   |   |   |   |   |   |   |   |   |   |   |   |   |    |  |    |    |    |
        * |------------------------------------------------------------|  |--------------|
        * |     |   |   |   |RST|   |   |   |   |   |   |   |   |      |  |    |    |    |
        * |------------------------------------------------------------|  `--------------'
        * |       |   |   |   |   |   |   |   |   |   |   |   |  |     |
        * |------------------------------------------------------------|       ,----.
        * |     |  |   |   |   |   |   |   |   |Bl-|Bl+|    | Mute|    |       |Vol+|
        * |------------------------------------------------------------|  ,--------------.
        * |    |    |    |         BL_Toggle         |    |   |   |    |  |    |Vol-|    |
        * `------------------------------------------------------------'  `--------------'
        */
        [_FL1] = LAYOUT(
            _______,          _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, _______, _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
            _______, _______, _______, _______, RESET,   _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, BL_DEC,  BL_INC,  _______, KC_MUTE,          _______,          KC_VOLU,
            _______, _______, _______,                            BL_TOGG,                            _______, _______, _______,          _______, _______, KC_VOLD, _______),
        /* _FL2: Function Layer 2 - For ISO enter use ANSI enter
        * ,------------------------------------------------------------.  ,--------------.
        * |     |  |   |   |   |  |   |   |   |   |   |   |   |   |    |  |    |    |    |
        * |------------------------------------------------------------|  |--------------|
        * |   |   |   |   |   |   |   |   |   |   |   |   |   |   |    |  |    |    |    |
        * |------------------------------------------------------------|  |--------------|
        * |     |   |   |   |   |   |   |   |   |   |   |   |   |      |  |    |    |    |
        * |------------------------------------------------------------|  `--------------'
        * |       |   |   |   |   |   |   |   |   |   |   |   |  |     |
        * |------------------------------------------------------------|       ,----.
        * |     |  |   |   |   |   |   |   |   |   |   |    |     |    |       |    |
        * |------------------------------------------------------------|  ,--------------.
        * |    |    |    |                           |    |   |   |    |  |    |    |    |
        * `------------------------------------------------------------'  `--------------'
        */
        [_FL2] = LAYOUT(
            _______,          _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, _______, _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,          _______, 
            _______, _______, _______,                            _______,                            _______, _______, _______,          _______, _______, _______, _______),
        /* _FL3: Function Layer 3 - For ISO enter use ANSI enter
        * ,------------------------------------------------------------.  ,--------------.
        * |     |  |   |   |   |  |   |   |   |   |   |   |   |   |    |  |    |    |    |
        * |------------------------------------------------------------|  |--------------|
        * |   |   |   |   |   |   |   |   |   |   |   |   |   |   |    |  |    |    |    |
        * |------------------------------------------------------------|  |--------------|
        * |     |   |   |   |   |   |   |   |   |   |   |   |   |      |  |    |    |    |
        * |------------------------------------------------------------|  `--------------'
        * |       |   |   |   |   |   |   |   |   |   |   |   |  |     |
        * |------------------------------------------------------------|       ,----.
        * |     |  |   |   |   |   |   |   |   |   |   |    |     |    |       |    |
        * |------------------------------------------------------------|  ,--------------.
        * |    |    |    |                           |    |   |   |    |  |    |    |    |
        * `------------------------------------------------------------'  `--------------'
        */
        [_FL3] = LAYOUT(
            _______,          _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, _______, _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, _______, _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
            _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,          _______, 
            _______, _______, _______,                            _______,                            _______, _______, _______,          _______, _______, _______, _______),
    
    };

